import sys
import argparse
import logging
import time
import vcf

def call_gt(tag):
	if sum(tag) == 2:
		return '1/1'
	else:
		if tag[0] == 1:
			return '1/0'
		if tag[1] == 1:
			return '0/1'

	return './.'

def main_ctrl(args):

	fileout = open(args.outvcf, 'w')

	header = open(args.invcf, 'r')
	for line in header:
		if line[0] == '#':
			fileout.write(line)
		else:
			continue
	header.close()

	filein = vcf.Reader(open(args.invcf, 'r'))
	for record in filein:
		if len(record.FILTER) == 0:
			filter_table = 'PASS'
		else:
			filter_table = record.FILTER[0]
		tag = [0, 0]
		for i in record.INFO['RNAMES']:
			if 'cutesvh1' in i:
				tag[0] = 1
			if 'cutesvh2' in i:
				tag[1] = 1

		try:
			fileout.write("{chr}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{gt}\n".format(
				chr = record.CHROM,
				pos = record.POS,
				id = record.ID,
				ref = record.REF,
				alt = record.ALT[0],
				qual = record.QUAL,
				filter = filter_table,
				info = "SVTYPE=%s;SVLEN=%d;END=%d;RE=%d;RNAMES=%s"%(record.INFO['SVTYPE'], 
					record.INFO['SVLEN'], 
					record.INFO['END'], 
					record.INFO['RE'], 
					','.join(record.INFO['RNAMES'])),
				format = 'GT',
				# gt = '1/1' if sum(tag) == 2 else '0/1'
				gt = call_gt(tag)
				))
		except:
			if 'TRA' in record.INFO['SVTYPE'] or 'BND' in record.INFO['SVTYPE']:
				fileout.write("{chr}\t{pos}\t{id}\t{ref}\t{alt}\t{qual}\t{filter}\t{info}\t{format}\t{gt}\n".format(
					chr = record.CHROM,
					pos = record.POS,
					id = record.ID,
					ref = record.REF,
					alt = record.ALT[0],
					qual = record.QUAL,
					filter = filter_table,
					info = "SVTYPE=%s;RE=%d;RNAMES=%s"%(record.INFO['SVTYPE'], 
						record.INFO['RE'], 
						','.join(record.INFO['RNAMES'])),
					format = 'GT',
					# gt = '1/1' if sum(tag) == 2 else '0/1'
					gt = call_gt(tag)
				))
			else:
				pass
	fileout.close()

def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

USAGE="""\
	Convert the typical SV callsets generated from cuteSV to diploid based SV callsets.
	Author: Tao Jiang
	Email: tjiang@hit.edu.cn
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="diploid_calling.py", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	parser.add_argument("invcf", type=str, help="A vcf file generated from cuteSV.")
	parser.add_argument("outvcf", type=str, help="The output diploid based SV callsets.")
	args = parser.parse_args(argv)
	return args

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])




import numpy as np
from cuteSV.cuteSV_genotype import cal_CIPOS, overlap_cover, assign_gt
import logging

'''
*******************************************
                TO DO LIST
*******************************************
    1. Identify DP with samfile pointer;
    2. Add CIPOS, CILEN and/or CIEND;
    3. Determine (IM)PRECISE type.
*******************************************

'''

def resolution_DEL(path, chr, svtype, read_count, threshold_gloab, max_cluster_bias,
                 minimum_support_reads, bam_path, action, gt_round, remain_reads_ratio):

    '''
    cluster DEL
    ********************************************************************************************
    path:	DEL.sigs
    chr:	chromosome id
    svtype:	<DEL>
    
    SEQTYPE		read_count 	max_cluster_bias 	sv_size		threshold_gloab 	threshold_local 
    --------------------------------------------------------------------------------------------
    CCS			3			200 bp (<500 bp)	30 bp 		0.4					0.5
    CLR			5/10		200 bp (<500 bp)	50 bp 		0.3					0.7
    --------------------------------------------------------------------------------------------
    
    Input file format
    --------------------------------------------------------------------------------------------
    column	#1	#2	#3	#4	#5
            DEL	CHR	BP	LEN	ID	
    #1	deletion type
    #2	chromosome number
    #3	breakpoint in each read
    #4	DEL_len in each read
    #5	read ID
    ********************************************************************************************
    '''
    if remain_reads_ratio > 1:
        remain_reads_ratio = 1
    semi_del_cluster = list()
    semi_del_cluster.append([0,0,''])
    candidate_single_SV = list()

    file = open("%s%s.sigs"%(path, "DEL"), 'r')
    for line in file:
        seq = line.strip('\n').split('\t')
        if seq[1] != chr:
            continue

        pos = int(seq[2])
        indel_len = int(seq[3])
        read_id = seq[4]
        
        if pos - semi_del_cluster[-1][0] > max_cluster_bias:
            if len(semi_del_cluster) >= read_count:
                if semi_del_cluster[-1][0] == semi_del_cluster[-1][1] == 0:
                    pass
                else:
                    generate_del_cluster(semi_del_cluster, 
                                        chr, 
                                        svtype, 
                                        read_count, 
                                        threshold_gloab, 
                                        # threshold_local, 
                                        minimum_support_reads, 
                                        candidate_single_SV,
                                        action,
                                        gt_round,
                                        remain_reads_ratio)
            semi_del_cluster = []
            semi_del_cluster.append([pos, indel_len, read_id])
        else:
            if semi_del_cluster[-1][0] == semi_del_cluster[-1][1] == 0:
                semi_del_cluster = []
                semi_del_cluster.append([pos, indel_len, read_id])
            else:
                semi_del_cluster.append([pos, indel_len, read_id])

    if len(semi_del_cluster) >= read_count:
        if semi_del_cluster[-1][0] == semi_del_cluster[-1][1] == 0:
            pass
        else:
            generate_del_cluster(semi_del_cluster, 
                                chr, 
                                svtype, 
                                read_count, 
                                threshold_gloab, 
                                # threshold_local, 
                                minimum_support_reads, 
                                candidate_single_SV,
                                action,
                                gt_round,
                                remain_reads_ratio)
    file.close()
    if action:
        candidate_single_SV_gt = call_gt(path, chr, candidate_single_SV, max_cluster_bias, 'DEL')
        logging.info("Finished %s:%s."%(chr, "DEL"))
        return candidate_single_SV_gt
    else:
        logging.info("Finished %s:%s."%(chr, "DEL"))
        return candidate_single_SV

def generate_del_cluster(semi_del_cluster, chr, svtype, read_count, 
    threshold_gloab, minimum_support_reads, candidate_single_SV, 
    action, gt_round, remain_reads_ratio):

    '''
    generate deletion
    *************************************************************
    threshold_gloab 	threshold_local 	minimum_support_reads
    -------------------------------------------------------------
        0.3					0.7 					5		CLR
        0.4					0.5 				  <=5		CCS
    *************************************************************
    '''

    # Remove duplicates
    read_tag = dict()
    for element in semi_del_cluster:
        if element[2] not in read_tag:
            read_tag[element[2]] = element
        else:
            if element[1] > read_tag[element[2]][1]:
                read_tag[element[2]] = element

    if len(read_tag) < read_count:
        return

    read_tag2SortedList = sorted(list(read_tag.values()), key = lambda x:x[1])
    global_len = [i[1] for i in read_tag2SortedList]
    DISCRETE_THRESHOLD_LEN_CLUSTER_DEL_TEMP = threshold_gloab * np.mean(global_len)

    last_len = read_tag2SortedList[0][1]

    allele_collect = list()
    '''
    *************************************************************
        #1 				#2			#3			#4
    -------------------------------------------------------------
        del-breakpoint	del-len		#support 	read-id
    *************************************************************
    '''
    allele_collect.append([[read_tag2SortedList[0][0]],[read_tag2SortedList[0][1]],[],
        [read_tag2SortedList[0][2]]])

    for i in read_tag2SortedList[1:]:
        if i[1] - last_len > DISCRETE_THRESHOLD_LEN_CLUSTER_DEL_TEMP:
            allele_collect[-1][2].append(len(allele_collect[-1][0]))
            allele_collect.append([[],[],[],[]])

        allele_collect[-1][0].append(i[0])
        allele_collect[-1][1].append(i[1])
        allele_collect[-1][3].append(i[2])
        last_len = i[1]
    allele_collect[-1][2].append(len(allele_collect[-1][0]))
    allele_sort = sorted(allele_collect, key = lambda x:x[2])

    for allele in allele_sort:
        if allele[2][0] >= minimum_support_reads:
            allele_list = list()
            var_list = list()
            remain_allele_num = max(int(remain_reads_ratio * allele[2][0]), 1)
            pos_mean = np.mean(allele[0])
            for i in range(len(allele[0])):
                var_list.append((abs(allele[0][i] - pos_mean), i))
            var_list.sort(key=lambda x:x[0])
            for i in range(remain_allele_num):
                allele_list.append(allele[0][var_list[i][1]])
            breakpointStart = np.mean(allele_list)
            search_threshold = allele_list[0]

            allele_list = list()
            var_list = list()
            len_mean = np.mean(allele[1])
            for i in range(len(allele[1])):
                var_list.append((abs(allele[1][i] - len_mean), i))
            var_list.sort(key=lambda x:x[0])
            for i in range(remain_allele_num):
                allele_list.append(allele[1][var_list[i][1]])
            signalLen = np.mean(allele_list)

            # breakpointStart = np.mean(allele[0])
            # search_threshold = np.min(allele[0])
            CIPOS = cal_CIPOS(np.std(allele[0]), len(allele[0]))
            # signalLen = np.mean(allele[1])
            signalLen_STD = np.std(allele[1])
            CILEN = cal_CIPOS(np.std(allele[1]), len(allele[1]))

            if action:
                candidate_single_SV.append([chr, 
                                            svtype, 
                                            int(breakpointStart), 
                                            int(-signalLen), 
                                            allele[2][0], 
                                            str(CIPOS),
                                            str(CILEN),
                                            int(search_threshold),
                                            allele[3]])
            else:
                candidate_single_SV.append([chr, 
                                            svtype, 
                                            str(int(breakpointStart)), 
                                            str(int(-signalLen)), 
                                            str(allele[2][0]), 
                                            str(CIPOS),
                                            str(CILEN),
                                            '.',
                                            './.',
                                            '.,.,.',
                                            '.',
                                            '.',
                                            str(','.join(allele[3]))])
    

def resolution_INS(path, chr, svtype, read_count, threshold_gloab, 
    max_cluster_bias, minimum_support_reads, bam_path, action, gt_round, remain_reads_ratio):
    
    '''
    cluster INS
    ********************************************************************************************
    path:	INS.sigs
    chr:	chromosome id
    svtype:	<INS>
    
    SEQTYPE		read_count 	max_cluster_bias 	sv_size		threshold_gloab 	threshold_local 
    --------------------------------------------------------------------------------------------
    CCS			3			200 bp (<500 bp)	30 bp 		0.65				0.7
    CLR			5/10		100 bp (<500 bp)	50 bp 		0.2					0.6
    --------------------------------------------------------------------------------------------
    
    Input file format
    --------------------------------------------------------------------------------------------
    column	#1	#2	#3	#4	#5 #6
            INS	CHR	BP	LEN	ID SEQ
    #1	insertion type
    #2	chromosome number
    #3	breakpoint in each read
    #4	INS_len in each read
    #5	read ID
    #6  INS sequence
    ********************************************************************************************
    '''
    if remain_reads_ratio > 1:
        remain_reads_ratio = 1
    semi_ins_cluster = list()
    semi_ins_cluster.append([0,0,'',''])
    candidate_single_SV = list()

    file = open("%s%s.sigs"%(path, "INS"), 'r')
    for line in file:
        seq = line.strip('\n').split('\t')
        if seq[1] != chr:
            continue

        pos = int(seq[2])
        indel_len = int(seq[3])
        read_id = seq[4]
        try:
            ins_seq = seq[5]
        except:
            ins_seq = ''
        
        if pos - semi_ins_cluster[-1][0] > max_cluster_bias:
            if len(semi_ins_cluster) >= read_count:
                if semi_ins_cluster[-1][0] == semi_ins_cluster[-1][1] == 0:
                    pass
                else:
                    generate_ins_cluster(semi_ins_cluster, 
                                        chr, 
                                        svtype, 
                                        read_count, 
                                        threshold_gloab, 
                                        # threshold_local, 
                                        minimum_support_reads, 
                                        candidate_single_SV,
                                        action,
                                        gt_round,
                                        remain_reads_ratio)
            semi_ins_cluster = []
            semi_ins_cluster.append([pos, indel_len, read_id, ins_seq])
        else:
            if semi_ins_cluster[-1][0] == semi_ins_cluster[-1][1] == 0:
                semi_ins_cluster = []
                semi_ins_cluster.append([pos, indel_len, read_id, ins_seq])
            else:
                semi_ins_cluster.append([pos, indel_len, read_id, ins_seq])

    if len(semi_ins_cluster) >= read_count:
        if semi_ins_cluster[-1][0] == semi_ins_cluster[-1][1] == 0:
            pass
        else:
            generate_ins_cluster(semi_ins_cluster, 
                                chr, 
                                svtype, 
                                read_count, 
                                threshold_gloab, 
                                # threshold_local, 
                                minimum_support_reads, 
                                candidate_single_SV,
                                action,
                                gt_round,
                                remain_reads_ratio)
    file.close()
    if action:
        candidate_single_SV_gt = call_gt(path, chr, candidate_single_SV, 1000, 'INS') # max_cluster_bias
        logging.info("Finished %s:%s."%(chr, "INS"))
        return candidate_single_SV_gt
    else:
        logging.info("Finished %s:%s."%(chr, "INS"))
        return candidate_single_SV

def generate_ins_cluster(semi_ins_cluster, chr, svtype, read_count, 
    threshold_gloab, minimum_support_reads, candidate_single_SV, 
    action, gt_round, remain_reads_ratio):
        
    '''
    generate insertion
    *************************************************************
    threshold_gloab 	threshold_local 	minimum_support_reads
    -------------------------------------------------------------
        0.2					0.6 					5		CLR
        0.65				0.7 				  <=5		CCS
    *************************************************************
    '''
    # Remove duplicates
    read_tag = dict()
    for element in semi_ins_cluster:
        if element[2] not in read_tag:
            read_tag[element[2]] = element
        else:
            if element[1] > read_tag[element[2]][1]:
                read_tag[element[2]] = element

    if len(read_tag) < read_count:
        return

    read_tag2SortedList = sorted(list(read_tag.values()), key = lambda x:x[1])
    # start&end breakpoint
    global_len = [i[1] for i in read_tag2SortedList]
    DISCRETE_THRESHOLD_LEN_CLUSTER_INS_TEMP = threshold_gloab * np.mean(global_len)
    last_len = read_tag2SortedList[0][1]

    allele_collect = list()
    allele_collect.append([[read_tag2SortedList[0][0]],
                            [read_tag2SortedList[0][1]],
                            [], 
                            [read_tag2SortedList[0][2]],
                            [read_tag2SortedList[0][3]]])

    for i in read_tag2SortedList[1:]:
        if i[1] - last_len > DISCRETE_THRESHOLD_LEN_CLUSTER_INS_TEMP:
            allele_collect[-1][2].append(len(allele_collect[-1][0]))
            allele_collect.append([[],[],[],[],[]])

        allele_collect[-1][0].append(i[0])
        allele_collect[-1][1].append(i[1])
        allele_collect[-1][3].append(i[2])
        allele_collect[-1][4].append(i[3])
        last_len = i[1]
    allele_collect[-1][2].append(len(allele_collect[-1][0]))
    allele_sort = sorted(allele_collect, key = lambda x:x[2])

    for allele in allele_sort:
        if allele[2][0] >= minimum_support_reads:
            allele_list = list()
            var_list = list()
            remain_allele_num = max(int(remain_reads_ratio * allele[2][0]), 1)
            pos_mean = np.mean(allele[0])
            for i in range(len(allele[0])):
                var_list.append((abs(allele[0][i] - pos_mean), i))
            var_list.sort(key=lambda x:x[0])
            for i in range(remain_allele_num):
                allele_list.append(allele[0][var_list[i][1]])
            breakpointStart = np.mean(allele_list)

            allele_list = list()
            var_list = list()
            len_mean = np.mean(allele[1])
            for i in range(len(allele[1])):
                var_list.append((abs(allele[1][i] - len_mean), i))
            var_list.sort(key=lambda x:x[0])
            for i in range(remain_allele_num):
                allele_list.append(allele[1][var_list[i][1]])
            signalLen = np.mean(allele_list)

            # breakpointStart = np.mean(allele[0])
            CIPOS = cal_CIPOS(np.std(allele[0]), len(allele[0]))
            # signalLen = np.mean(allele[1])
            signalLen_STD = np.std(allele[1])
            CILEN = cal_CIPOS(np.std(allele[1]), len(allele[1]))
            ideal_ins_seq = '<INS>'
            for pos,i in zip(allele[0],allele[4]):
                if len(i) >= int(signalLen):
                    breakpointStart = pos
                    ideal_ins_seq = i[0:int(signalLen)]
                    break
            if ideal_ins_seq == '<INS>':
                continue

            if action:
                candidate_single_SV.append([chr, 
                                            svtype, 
                                            int(breakpointStart), 
                                            int(signalLen), 
                                            allele[2][0], 
                                            str(CIPOS),
                                            str(CILEN),
                                            int(breakpointStart), 
                                            allele[3],
                                            ideal_ins_seq])
            else:
                candidate_single_SV.append([chr, 
                                            svtype, 
                                            str(int(breakpointStart)), 
                                            str(int(signalLen)), 
                                            str(allele[2][0]), 
                                            str(CIPOS),
                                            str(CILEN),
                                            '.',
                                            './.',
                                            '.,.,.',
                                            ".",
                                            ".",
                                            str(','.join(allele[3])),
                                            ideal_ins_seq])


def run_del(args):
    return resolution_DEL(*args)

def run_ins(args):
    return resolution_INS(*args)

def call_gt(temporary_dir, chr, candidate_single_SV, max_cluster_bias, svtype):
    reads_list = list() # [(10000, 10468, 0, 'm54238_180901_011437/52298335/ccs'), ...]
    readsfile = open("%sreads.sigs"%(temporary_dir), 'r')
    for line in readsfile:
        seq = line.strip().split('\t')
        if seq[0] != chr:
            continue
        reads_list.append([int(seq[1]), int(seq[2]), int(seq[3]), seq[4]])
    readsfile.close()
    svs_list = list()
    for item in candidate_single_SV:
        svs_list.append((max(item[7] - max_cluster_bias, 0), item[7] + max_cluster_bias))
    iteration_dict, primary_num_dict, cover_dict = overlap_cover(svs_list, reads_list) # both key(sv idx), value(set(read id))
    assert len(cover_dict) == len(candidate_single_SV), "overlap length error"

    read_id_dict = dict()
    for i in range(len(candidate_single_SV)):
        read_id_dict[i] = candidate_single_SV[i][8]
    assign_list = assign_gt(iteration_dict, primary_num_dict, cover_dict, read_id_dict)
    # [[DV, DR, GT, GL, GQ, QUAL] ...]
    assert len(candidate_single_SV) == len(assign_list), "assign error"
    candidate_single_SV_gt = list()
    for i in range(len(candidate_single_SV)):
        assert len(assign_list[i]) == 6, "assign genotype error"
        candidate_single_SV_gt.append([candidate_single_SV[i][0], 
                                    candidate_single_SV[i][1], 
                                    str(candidate_single_SV[i][2]), 
                                    str(candidate_single_SV[i][3]), 
                                    str(candidate_single_SV[i][4]), 
                                    candidate_single_SV[i][5],
                                    candidate_single_SV[i][6],
                                    str(assign_list[i][1]),
                                    str(assign_list[i][2]),
                                    str(assign_list[i][3]),
                                    str(assign_list[i][4]),
                                    str(assign_list[i][5]),
                                    ','.join(candidate_single_SV[i][8])])
        if svtype == 'INS':
            candidate_single_SV_gt[i].append(candidate_single_SV[i][9])
    return candidate_single_SV_gt


import sys
import argparse
import logging
import time

def pase_info(seq):
	info = {'SVLEN': 0, 'END': 0, "SVTYPE": '', "RE": 0, "CHR2": ''}
	for i in seq.split(';'):
		if i.split('=')[0] in ["SVLEN", "END", "RE"]:
			try:
				info[i.split('=')[0]] = abs(int(float(i.split('=')[1])))
			except:
				pass
		if i.split('=')[0] in ["CHR2"]:
			info[i.split('=')[0]] = i.split('=')[1]
		if i.split('=')[0] in ["SVTYPE"]:
			info[i.split('=')[0]] = i.split('=')[1][0:3]

	return info

def phase_GT(seq):
	i = seq.split(':')
	if i[0] in ['0/1', '1/0']:
		return 'het'
	elif i[0] == '1/1':
		return 'hom'
	else:
		return 'unknown'

def load_callset(path):
	callset = dict()
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		info = pase_info(seq[7])

		if info['SVTYPE'] in ['DEL', 'INS', 'DUP', 'INV']:
			if info['SVTYPE'] not in callset:
				callset[info['SVTYPE']] = dict()
			if info['SVLEN'] == 0:
				info['SVLEN'] = info['END'] - pos + 1

			if chr not in callset[info['SVTYPE']]:
				callset[info['SVTYPE']][chr] = list()
			callset[info['SVTYPE']][chr].append([pos, info['END'], info['SVLEN'], phase_GT(seq[9]), [0, 0]])

		if info['SVTYPE'] == "BND":
			if seq[4][0] == ']':
				form = ']]N'
				chr2 = seq[4].split(':')[0][1:]
				pos2 = int(seq[4].split(':')[1][:-2])
			elif seq[4][0] == '[':
				form = '[[N'
				chr2 = seq[4].split(':')[0][1:]
				pos2 = int(seq[4].split(':')[1][:-2])
			else:
				if seq[4][1] == ']':
					form = 'N]]'
					chr2 = seq[4].split(':')[0][2:]
					pos2 = int(seq[4].split(':')[1][:-1])
				else:
					form = 'N[['
					chr2 = seq[4].split(':')[0][2:]
					pos2 = int(seq[4].split(':')[1][:-1])
			if info['SVTYPE'] not in callset:
				callset[info['SVTYPE']] = dict()
			if info['END'] == 0:
				info['CHR2'] = chr2
				info['END'] = pos2

			if chr not in callset[info['SVTYPE']]:
				callset[info['SVTYPE']][chr] = list()
			callset[info['SVTYPE']][chr].append([pos, info['CHR2'], info['END'], form, phase_GT(seq[9]), [0, 0]])

	file.close()
	return callset

def eva_record(call_A, call_B, bias, offect, tag1, tag2):
	# call_A 0/1
	# call_B 1/1
	for svtype in call_A:
		if svtype not in call_B:
			continue

		for chr in call_A[svtype]:
			if chr not in call_B[svtype]:
				continue

			for i in call_A[svtype][chr]:
				for j in call_B[svtype][chr]:

					if svtype == 'INS':
						if abs(i[0]-j[0]) <= offect and float(min(i[2],j[2])/max(i[2],j[2])) >= bias:
							i[-1][tag1] = 1
							j[-1][tag2] = 1
					elif svtype == 'BND':
						if i[1] == j[1] and i[3] == j[3]:
							if abs(i[0]-j[0]) <= offect and abs(i[2]-j[2]) <= offect:
								i[-1][tag1] = 1
								j[-1][tag2] = 1
					else:
						if max(i[0]-offect, j[0]) <= min(i[1]+offect, j[1]) and float(min(i[2],j[2])/max(i[2],j[2])) >= bias:
							i[-1][tag1] = 1
							j[-1][tag2] = 1

def statistics(callset, a, b, c):
	for svtype in callset:
		record = 0
		record_000 = 0
		record_010 = 0
		record_001 = 0
		record_011 = 0
		for chr in callset[svtype]:
			for i in callset[svtype][chr]:
				record += 1
				if i[-1][0] == 0 and i[-1][1] == 0:
					record_000 += 1
				if i[-1][0] == 1 and i[-1][1] == 0:
					record_010 += 1
				if i[-1][0] == 0 and i[-1][1] == 1:
					record_001 += 1
				if i[-1][0] == 1 and i[-1][1] == 1:
					record_011 += 1

		logging.info("%s number of %s:\t%d"%(svtype, a, record))
		logging.info("Only %s:\t%d"%(a, record_000))
		logging.info("%s and %s:\t%d"%(a, b, record_010))
		logging.info("%s and %s:\t%d"%(a, c, record_001))
		logging.info("%s and %s and %s:\t%d"%(a, b, c, record_011))


def main_ctrl(args):	
	logging.info("Load SV callset of selected caller.")

	clr_callset = load_callset(args.c1)
	ont_callset = load_callset(args.c2)
	ccs_callset = load_callset(args.c3)

	logging.info("Comparing...")
	eva_record(clr_callset, ont_callset, args.bias, args.offect, 0, 0)
	eva_record(clr_callset, ccs_callset, args.bias, args.offect, 1, 0)
	eva_record(ont_callset, ccs_callset, args.bias, args.offect, 1, 1)

	logging.info("Final results:")
	statistics(clr_callset, "CLR", "ONT", "CCS")
	statistics(ont_callset, "ONT", "CLR", "CCS")
	statistics(ccs_callset, "CCS", "CLR", "ONT")


def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	# print args
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

USAGE="""\
	Evaluate SV callset generated by cuteSV
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="Trio_eval", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	parser.add_argument("c1", type=str, help="PacBio callset")
	parser.add_argument('c2', type=str, help = "ONT callset")
	parser.add_argument('c3', type=str, help = "High confidence callset")
	parser.add_argument('-b', '--bias', help = "Bias of overlaping.[%(default)s]", default = 0.7, type = float)
	parser.add_argument('-o', '--offect', help = "Offect of translocation overlaping.[%(default)s]", default = 1000, type = int)
	args = parser.parse_args(argv)
	return args

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])


import numpy as np
import logging
from cuteSV.cuteSV_genotype import overlap_cover, assign_gt

'''
*******************************************
                TO DO LIST
*******************************************
    1. Identify DP with samfile pointer;
    2. Add CIPOS, CILEN and/or CIEND;
    3. Determine (IM)PRECISE type.
    4. Filter DUP to improve INS FN rate.
*******************************************
'''

def resolution_DUP(path, chr, read_count, max_cluster_bias, sv_size, 
    bam_path, action, MaxSize, gt_round):
    semi_dup_cluster = list()
    semi_dup_cluster.append([0, 0, ''])
    candidate_single_SV = list()

    file = open("%s%s.sigs"%(path, "DUP"), 'r')
    for line in file:
        seq = line.strip('\n').split('\t')
        if seq[1] != chr:
            continue

        pos_1 = int(seq[2])
        pos_2 = int(seq[3])
        read_id = seq[4]
        
        if pos_1 - semi_dup_cluster[-1][0] > max_cluster_bias or pos_2 - semi_dup_cluster[-1][1] > max_cluster_bias:
            if len(semi_dup_cluster) >= read_count:
                if semi_dup_cluster[-1][0] == semi_dup_cluster[-1][1] == 0:
                    pass
                else:
                    generate_dup_cluster(semi_dup_cluster, 
                                        chr, 
                                        read_count, 
                                        max_cluster_bias, 
                                        sv_size, 
                                        candidate_single_SV,
                                        action,
                                        MaxSize,
                                        gt_round)
            semi_dup_cluster = []
            semi_dup_cluster.append([pos_1, pos_2, read_id])
        else:
            if semi_dup_cluster[-1][0] == semi_dup_cluster[-1][1] == 0:
                semi_dup_cluster = []
                semi_dup_cluster.append([pos_1, pos_2, read_id])
            else:
                semi_dup_cluster.append([pos_1, pos_2, read_id])

    if len(semi_dup_cluster) >= read_count:
        if semi_dup_cluster[-1][0] == semi_dup_cluster[-1][1] == 0:
            pass
        else:
            generate_dup_cluster(semi_dup_cluster, 
                                chr, 
                                read_count, 
                                max_cluster_bias, 
                                sv_size, 
                                candidate_single_SV,
                                action,
                                MaxSize,
                                gt_round)
    file.close()
    if action:
        candidate_single_SV_gt = call_gt(path, chr, candidate_single_SV, max_cluster_bias)
        logging.info("Finished %s:%s."%(chr, "DUP"))
        return candidate_single_SV_gt
    else:
        logging.info("Finished %s:%s."%(chr, "DUP"))
        return candidate_single_SV

def generate_dup_cluster(semi_dup_cluster, chr, read_count, max_cluster_bias, 
    sv_size, candidate_single_SV, action, MaxSize, gt_round):
    # calculate support reads
    support_read = list(set([i[2] for i in semi_dup_cluster]))
    if len(support_read) < read_count:
        return

    low_b = int(len(semi_dup_cluster)*0.4)
    up_b = int(len(semi_dup_cluster)*0.6)

    if low_b == up_b:
        breakpoint_1 = semi_dup_cluster[low_b][0]
        breakpoint_2 = semi_dup_cluster[low_b][1]
    else:
        breakpoint_1 = [i[0] for i in semi_dup_cluster[low_b:up_b]]
        breakpoint_2 = [i[1] for i in semi_dup_cluster[low_b:up_b]]
        breakpoint_1 = int(sum(breakpoint_1)/len(semi_dup_cluster[low_b:up_b]))
        breakpoint_2 = int(sum(breakpoint_2)/len(semi_dup_cluster[low_b:up_b]))


    if sv_size <= breakpoint_2 - breakpoint_1 <= MaxSize or (sv_size <= breakpoint_2 - breakpoint_1 and MaxSize == -1):
        if action:
            candidate_single_SV.append([chr,
                                        'DUP', 
                                        breakpoint_1, 
                                        breakpoint_2,
                                        support_read])
            # print("DUP", chr, int(breakpoint_1), int(breakpoint_2), DR, DV, QUAL, "%.4f"%cost_time)
        else:
            candidate_single_SV.append([chr,
                                        'DUP', 
                                        str(breakpoint_1), 
                                        str(breakpoint_2 - breakpoint_1), 
                                        str(len(support_read)),
                                        '.',
                                        './.',
                                        '.,.,.',
                                        '.',
                                        '.',
                                        str(','.join(support_read))])


def run_dup(args):
    return resolution_DUP(*args)

def call_gt(temporary_dir, chr, candidate_single_SV, max_cluster_bias):
    reads_list = list() # [(10000, 10468, 0, 'm54238_180901_011437/52298335/ccs'), ...]
    readsfile = open("%sreads.sigs"%(temporary_dir), 'r')
    for line in readsfile:
        seq = line.strip().split('\t')
        if seq[0] != chr:
            continue
        reads_list.append([int(seq[1]), int(seq[2]), int(seq[3]), seq[4]])
    readsfile.close()
    svs_list = list()
    for item in candidate_single_SV:
        new_cluster_bias = min(max_cluster_bias, item[3] - item[2])
        svs_list.append((max(item[2] - new_cluster_bias/2, 0), item[2] + new_cluster_bias/2))
    for item in candidate_single_SV:
        new_cluster_bias = min(max_cluster_bias, item[3] - item[2])
        svs_list.append((max(item[3] - new_cluster_bias/2, 0), item[3] + new_cluster_bias/2))
    iteration_dict, primary_num_dict, cover_dict = overlap_cover(svs_list, reads_list) # both key(sv idx), value(set(read id))
    assert len(cover_dict) == 2 * len(candidate_single_SV), "overlap length error"
    candidate_single_SV_length = len(candidate_single_SV)
    for idx in range(candidate_single_SV_length):
        for item in cover_dict[idx + candidate_single_SV_length]:
            cover_dict[idx].add(item)
    for idx in range(candidate_single_SV_length, candidate_single_SV_length * 2, 1):
        cover_dict.pop(idx)
    assert len(cover_dict) == len(candidate_single_SV), "overlap length error"

    read_id_dict = dict()
    for i in range(len(candidate_single_SV)):
        read_id_dict[i] = candidate_single_SV[i][4]
    assign_list = assign_gt(iteration_dict, primary_num_dict, cover_dict, read_id_dict)
    # [[DV, DR, GT, GL, GQ, QUAL] ...]
    assert len(candidate_single_SV) == len(assign_list), "assign error"
    candidate_single_SV_gt = list()
    for i in range(len(candidate_single_SV)):
        candidate_single_SV_gt.append([candidate_single_SV[i][0], 
                                    candidate_single_SV[i][1], 
                                    str(candidate_single_SV[i][2]), 
                                    str(candidate_single_SV[i][3] - candidate_single_SV[i][2]), 
                                    str(len(candidate_single_SV[i][4])),
                                    str(assign_list[i][1]),
                                    str(assign_list[i][2]),
                                    str(assign_list[i][3]),
                                    str(assign_list[i][4]),
                                    str(assign_list[i][5]),
                                    ','.join(candidate_single_SV[i][4])])
    return candidate_single_SV_gt	


from cuteSV.cuteSV_genotype import cal_CIPOS, overlap_cover, assign_gt
from multiprocessing import Pool
from pysam import VariantFile
import math
import time
import logging
import numpy as np

def parse_svtype(sv_type):
    if 'DEL' in sv_type:
        return 'DEL'
    if 'INS' in sv_type:
        return 'INS'
    if 'INV' in sv_type:
        return 'INV'
    if 'DUP' in sv_type:
        return 'DUP'
    if 'BND' in sv_type or 'TRA' in sv_type:
        return 'TRA'
    return 'NA'

def parse_to_int(sth):
    if sth == None:
        return 0
    elif isinstance(sth, str):
        return int(sth)
    elif isinstance(sth, list):
        return parse_to_int(sth[0])
    elif isinstance(sth, tuple):
        return parse_to_int(sth[0])
    elif isinstance(sth, int):
        return sth
    else:
        return sth

# INDEL end->len, others:end
def parse_record(record):
    sv_type = parse_svtype(record.info['SVTYPE'])
    chrom1 = record.chrom
    start = parse_to_int(record.pos)
    chrom2 = ''
    svlen = 0
    end = 0 # len for indel, end for others
    if 'SVLEN' in record.info:
        svlen = abs(parse_to_int(record.info['SVLEN']))
    if 'END' in record.info:
        end = parse_to_int(record.info['END'])
    try:
        end = parse_to_int(record.stop)
    except:
        pass
    if sv_type == 'INS' or sv_type == 'DEL':
        end = svlen
    else:
        if start == end:
            end = start + svlen
    if sv_type == 'TRA':
        if 'CHR2' in record.info:
            chrom2 = record.info['CHR2']
        if 'END' in record.info:
            end = parse_to_int(record.info['END'])
        try:
            tra_alt = str(record.alts[0])
            if tra_alt[0] == 'N':
                if tra_alt[1] == '[':
                    tra_type = 'A'
                else:
                    tra_type = 'B'
            elif tra_alt[0] == '[':
                tra_type = 'C'
            else:
                tra_type = 'D'
            if tra_alt[0] == 'N':
                tra_alt = tra_alt[2:-1]
            else:
                tra_alt = tra_alt[1:-2]
            if ':' in tra_alt:
                chrom2 = tra_alt.split(':')[0]
                end = int(tra_alt.split(':')[1])
        except:
            pass
    strand = '.'
    if sv_type != 'TRA':
        chrom2 = record.chrom
        if 'STRAND' in record.info:
            strand = record.info['STRAND']
        elif 'STRANDS' in record.info:
            strand = record.info['STRANDS']
    if isinstance(strand, tuple) or isinstance(strand, list):
        strand = strand[0]
    svid = record.id
    ref = record.ref
    alts = record.alts[0]
    if 'SEQ' in record.info:
        if record.info['SVTYPE'] == 'INS' and record.alts[0] == '<INS>':
            alts = record.info['SEQ']
        if record.info['SVTYPE'] == 'DEL' and record.alts[0] == '<DEL>':
            ref = record.info['SEQ']
    return sv_type, chrom1, chrom2, start, end, strand, svid, ref, alts

def parse_sigs_chrom(var_type, work_dir, chrom_list):
    if var_type == 'DEL' or var_type == 'DUP':
        var_dict = dict()  #var_dict[chrom] = [chrom, start, len/end, read_id]
        with open(work_dir + var_type + '.sigs', 'r') as f:
            for line in f:
                seq = line.strip().split('\t')
                if seq[1] in chrom_list:
                    if seq[1] not in var_dict:
                        var_dict[seq[1]] = []
                    var_dict[seq[1]].append([seq[1], int(seq[2]), int(seq[3]), seq[4]])
        return var_dict
    if var_type == 'INS':
        var_dict = dict()  #var_dict[chrom] = [chrom, start, len, read_id, seq]
        with open(work_dir + 'INS.sigs', 'r') as f:
            for line in f:
                seq = line.strip().split('\t')
                if seq[1] in chrom_list:
                    if len(seq) < 6:
                        cigar = '<INS>'
                    else:
                        cigar = seq[5]
                    cigar = '<INS>'
                    if seq[1] not in var_dict:
                        var_dict[seq[1]] = []
                    var_dict[seq[1]].append([seq[1], int(seq[2]), int(seq[3]), seq[4], cigar])
        return var_dict
    if var_type == 'INV':
        var_dict = dict() # var_dict[chrom] = [chrom, start, end, read_id]
        with open(work_dir + 'INV.sigs', 'r') as f:
            for line in f:
                seq = line.strip().split('\t')
                if seq[1] in chrom_list:
                    chrom = seq[1]
                    if chrom not in var_dict:
                        var_dict[chrom] = []
                    var_dict[chrom].append([chrom, int(seq[3]), int(seq[4]), seq[5]])
        for chrom in var_dict:
            var_dict[chrom].sort(key=lambda x:x[1])
        return var_dict
    if var_type == 'TRA':
        var_dict = dict()  #var_dict[chrom1][chrom2] = [[chrom2, pos1, pos2, read_id]]
        with open(work_dir + 'TRA.sigs', 'r') as f:
            for line in f:
                seq = line.strip().split('\t')
                if seq[1] not in chrom_list:
                    continue
                chrom1 = seq[1]
                tra_type = seq[2]
                pos1 = int(seq[3])
                chrom2 = seq[4]
                pos2 = int(seq[5])
                read_id = seq[6]
                '''
                if chrom1 in var_dict:
                    if tra_type in var_dict[chrom1]:
                        if chrom2 in var_dict[chrom1][tra_type]:
                            var_dict[chrom1][tra_type][chrom2].append([chrom2, pos1, pos2, read_id])
                        else:
                            var_dict[chrom1][tra_type][chrom2] = [[chrom2, pos1, pos2, read_id]]
                    else:
                        var_dict[chrom1][tra_type] = dict()
                        var_dict[chrom1][tra_type][chrom2] = [[chrom2, pos1, pos2, read_id]]

                else:
                    var_dict[chrom1] = dict()
                    var_dict[chrom1][tra_type] = dict()
                    var_dict[chrom1][tra_type][chrom2] = [[chrom2, pos1, pos2, read_id]]
                '''
                if chrom1 not in var_dict:
                    var_dict[chrom1] = dict()
                if chrom2 not in var_dict[chrom1]:
                    var_dict[chrom1][chrom2] = []
                var_dict[chrom1][chrom2].append([chrom2, pos1, pos2, read_id])
        for chr1 in var_dict:
            for chr2 in var_dict[chr1]:
                var_dict[chr1][chr2].sort(key=lambda x:x[1])
        return var_dict

def check_same_variant(sv_type, end1, end2):
    if sv_type == 'INS' or sv_type == 'DEL':
        return 0.7 < min(end1, end2) / max(end1, end2) <= 1
    return abs(end1 - end2) < 1000

# var_list read_id which is similar to (pos, sv_end), the reads support the variant
def find_in_list(var_type, var_list, bias, pos, sv_end):
    '''
    if interval_start == 44058795:
        print('start find in list')
        print('interval_start=%d, interval_end=%d, pos=%d, sv_end=%d'%(interval_start, interval_end, pos, sv_end))
    '''
    if len(var_list) == 0:
        return [], 0
    left = 0
    right = len(var_list) - 1
    mid = 0
    while left < right:
        mid = (left + right) >> 1
        if var_list[mid][1] < pos:
            left = mid + 1
        else:
            right = mid
    read_id_list = set()
    search_start = -1
    search_end = -1
    if right > 0 and pos - var_list[right - 1][1] <= bias: ##
        for i in range(right - 1, -1, -1):
            if check_same_variant(var_type, var_list[i][2], sv_end):
                read_id_list.add(var_list[i][3])
                search_start = var_list[i][1]
            if i > 0 and (var_list[i][1] - var_list[i - 1][1] > bias or pos - var_list[i - 1][1] > 2000):
                break
    if var_list[right][1] - pos <= bias:
        for i in range(right, len(var_list)):
            if check_same_variant(var_type, var_list[i][2], sv_end):  # if abs(var_list[i][2] - sv_end) < 1000:
                read_id_list.add(var_list[i][3])
                search_end = var_list[i][1]
            if i < len(var_list) - 1 and (var_list[i + 1][1] - var_list[i][1] > bias or var_list[i + 1][1] - pos > 2000):
                break
    if search_start == -1:
        search_start = pos
    if search_end == -1:
        search_end = pos
    search_threshold = max(abs(pos - search_start), abs(pos - search_end))
    return list(read_id_list), search_threshold

def compare_len(len1, len2): # len1 < len2
    if len2 < 100:
        if len1 / len2 > 0.6:
            return True
    else:
        if len1 / len2 > 0.8:
            return True
    return False

def find_in_indel_list(var_type, var_list, bias, pos, sv_end, threshold_gloab):
    debug_pos = -1
    if len(var_list) == 0:
        return [], 0, '.,.', '.,.'
    left = 0
    right = len(var_list) - 1
    mid = 0
    while left < right:
        mid = (left + right) >> 1
        if var_list[mid][1] < pos:
            left = mid + 1
        else:
            right = mid
    candidates = []
    if right > 0 and pos - var_list[right - 1][1] <= bias:
        for i in range(right - 1, -1, -1):
            candidates.append(var_list[i]) # [chrom, start, len, read_id] or [chrom, start, len, read_id, seq]
            if i > 0 and (var_list[i][1] - var_list[i - 1][1] > bias or pos - var_list[i - 1][1] > 2 * bias):
                break
    if var_list[right][1] - pos <= bias:
        for i in range(right, len(var_list)):
            candidates.append(var_list[i])
            if i < len(var_list) - 1 and (var_list[i + 1][1] - var_list[i][1] > bias or var_list[i + 1][1] - pos > 2 * bias):
                break
    if len(candidates) == 0:
        return [], 0, '.,.', '.,.'
    read_tag = dict()
    for element in candidates:
        if element[3] not in read_tag:
            read_tag[element[3]] = []
        read_tag[element[3]].append(element)

    # merge sigs on the same read
    read_tag2SortedList = []
    for read_id in read_tag:
        for i in range(len(read_tag[read_id])):
            read_tag2SortedList.append(read_tag[read_id][i])
            for j in range(i + 1, len(read_tag[read_id]), 1):
                if var_type == 'DEL':
                    read_tag2SortedList.append([read_tag[read_id][i][0], int((read_tag[read_id][i][1]+read_tag[read_id][j][1])/2), read_tag[read_id][i][2]+read_tag[read_id][j][2], read_tag[read_id][i][3] ])
                else:
                    read_tag2SortedList.append([read_tag[read_id][i][0], int((read_tag[read_id][i][1]+read_tag[read_id][j][1])/2), read_tag[read_id][i][2]+read_tag[read_id][j][2], read_tag[read_id][i][3], read_tag[read_id][i][4] ])
    
    read_tag2SortedList = sorted(read_tag2SortedList, key = lambda x:x[2])
    global_len = [i[2] for i in read_tag2SortedList]
    DISCRETE_THRESHOLD_LEN_CLUSTER_TEMP = threshold_gloab * np.mean(global_len)

    if var_type == 'DEL':
        last_len = read_tag2SortedList[0][2]
        allele_collect = list()
        allele_collect.append([[read_tag2SortedList[0][1]],  # start
                                [read_tag2SortedList[0][2]],  # len
                                [],  # support
                                [read_tag2SortedList[0][3]]])  # read_id
        for i in read_tag2SortedList[1:]:
            if i[2] - last_len > DISCRETE_THRESHOLD_LEN_CLUSTER_TEMP:
                allele_collect[-1][2].append(len(allele_collect[-1][0]))
                #allele_collect[-1][4].append(np.mean(allele_collect[-1][1]))
                allele_collect.append([[],[],[],[]])

            allele_collect[-1][0].append(i[1])
            allele_collect[-1][1].append(i[2])
            allele_collect[-1][3].append(i[3])
            last_len = i[2]
        allele_collect[-1][2].append(len(allele_collect[-1][0]))
        allele_sort = sorted(allele_collect, key = lambda x:x[2])
        allele_idx = -1
        nearest_gap = 0x3f3f3f3f
        for i in range(len(allele_sort)):
            allele = allele_sort[i]
            signalLen = np.mean(allele[1])
            if abs(signalLen - sv_end) < nearest_gap:
                allele_idx = i
                nearest_gap = abs(signalLen - sv_end)
 
        if allele_idx == -1:
            read_id_set = set()
            CIPOS = "-0,0"
            CILEN = "-0,0"
            seq = '<DEL>'
            search_threshold = 0
        else:
            final_alleles = [[],[],[],[]]
            for i in range(len(allele_sort[allele_idx][0])):
                if min(allele_sort[allele_idx][1][i], sv_end) / max(allele_sort[allele_idx][1][i], sv_end) > 0.7:
                    final_alleles[0].append(allele_sort[allele_idx][0][i])
                    final_alleles[1].append(allele_sort[allele_idx][1][i])
                    final_alleles[3].append(allele_sort[allele_idx][3][i])
            if len(final_alleles[0]) == 0:
                read_id_set = set()
                CIPOS = "-0,0"
                CILEN = "-0,0"
                seq = '<DEL>'
                search_threshold = 0
            else:
                read_id_set = set(final_alleles[3])
                CIPOS = cal_CIPOS(np.std(final_alleles[0]), len(final_alleles[0]))
                CILEN = cal_CIPOS(np.std(final_alleles[1]), len(final_alleles[1]))
                seq = '<DEL>'
                search_start = min(final_alleles[0])
                search_end = max(final_alleles[0])
                search_threshold = max(abs(pos - search_start), abs(pos - search_end))
            
    else: # INS
        last_len = read_tag2SortedList[0][2]
        allele_collect = list()
        allele_collect.append([[read_tag2SortedList[0][1]],  # start
                                [read_tag2SortedList[0][2]],  # len
                                [],  # support
                                [read_tag2SortedList[0][3]],  # read_id
                                [read_tag2SortedList[0][4]]])  # ins_seq
        for i in read_tag2SortedList[1:]:
            if i[2] - last_len > DISCRETE_THRESHOLD_LEN_CLUSTER_TEMP:
                allele_collect[-1][2].append(len(allele_collect[-1][0]))
                allele_collect.append([[],[],[],[],[]])

            allele_collect[-1][0].append(i[1])
            allele_collect[-1][1].append(i[2])
            allele_collect[-1][3].append(i[3])
            allele_collect[-1][4].append(i[4])
            last_len = i[2]
        allele_collect[-1][2].append(len(allele_collect[-1][0]))
        allele_sort = sorted(allele_collect, key = lambda x:x[2])

        allele_idx = -1
        nearest_gap = 0x3f3f3f3f
        for i in range(len(allele_sort)):
            allele = allele_sort[i]
            signalLen = np.mean(allele[1])
            if abs(signalLen - sv_end) < nearest_gap:
                allele_idx = i
                nearest_gap = abs(signalLen - sv_end)

        if allele_idx == -1:
            read_id_set = set()
            CIPOS = "-0,0"
            CILEN = "-0,0"
            seq = '<INS>'
            search_threshold = 0
        else:
            final_alleles = [[],[],[],[],[]]
            for i in range(len(allele_sort[allele_idx][0])):
                if min(allele_sort[allele_idx][1][i], sv_end) / max(allele_sort[allele_idx][1][i], sv_end) > 0.7:
                    final_alleles[0].append(allele_sort[allele_idx][0][i])
                    final_alleles[1].append(allele_sort[allele_idx][1][i])
                    final_alleles[3].append(allele_sort[allele_idx][3][i])
                    final_alleles[4].append(allele_sort[allele_idx][4][i])

            if len(final_alleles[0]) == 0:
                read_id_set = set()
                CIPOS = "-0,0"
                CILEN = "-0,0"
                seq = '<INS>'
                search_threshold = 0
            else:
                if pos == debug_pos:
                    print(pos)
                read_id_set = set(final_alleles[3])
                CIPOS = cal_CIPOS(np.std(final_alleles[0]), len(final_alleles[0]))
                CILEN = cal_CIPOS(np.std(final_alleles[1]), len(final_alleles[1]))
                seq = '<INS>'
                #for i in final_alleles[4]:
                #    signalLen = np.mean(final_alleles[1])
                #    if len(i) >= int(signalLen):
                #        seq = i[0:int(signalLen)]
                if len(final_alleles[4][0]) >= sv_end:
                    seq = final_alleles[4][0][0:sv_end]
                search_start = min(final_alleles[0])
                search_end = max(final_alleles[0])
                search_threshold = max(abs(pos - search_start), abs(pos - search_end))
    return list(read_id_set), search_threshold, CIPOS, CILEN

def generate_dispatch(reads_count, chrom_list):
    dispatch = [[]]
    cur_count = 0
    for item in reads_count:
        if cur_count >= 10000:
            dispatch.append([])
            cur_count = 0
        cur_count = item[1]
        dispatch[-1].append(item[0])
    # chrom without reads
    for chrom in chrom_list:
        flag = 0
        for x in reads_count:
            if chrom == x[0]:
                flag = 1
        if flag == 0:
            dispatch[0].append(chrom)
    return dispatch

def force_calling_chrom(ivcf_path, temporary_dir, max_cluster_bias_dict, threshold_gloab_dict, gt_round, threads):
    logging.info('Check the parameter -Ivcf: OK.')
    logging.info('Enable to perform force calling.')

    # parse svs tobe genotyped
    vcf_reader = VariantFile(ivcf_path, 'r')
    svs_tobe_genotyped = dict()
    for record in vcf_reader.fetch():
        sv_type, chrom, sv_chr2, pos, sv_end, sv_strand, svid, ref, alts = parse_record(record)
        if sv_type not in ["DEL", "INS", "DUP", "INV", "TRA"]:
            continue
        if chrom not in svs_tobe_genotyped:
            svs_tobe_genotyped[chrom] = list()
        svs_tobe_genotyped[chrom].append([sv_type, sv_chr2, pos, sv_end, svid, ref, alts, sv_strand, chrom])
    
    # parse reads in alignment
    reads_count = dict()
    with open('%sreads.sigs'%(temporary_dir), 'r') as f:
        for line in f:
            seq = line.strip().split('\t')
            if seq[0] not in reads_count:
                reads_count[seq[0]] = 0
            reads_count[seq[0]] += 1
    reads_count = sorted(reads_count.items(), key=lambda x:x[1])
    dispatch = generate_dispatch(reads_count, svs_tobe_genotyped.keys())
    
    # force calling
    pool_result = list()
    result = list()
    process_pool = Pool(processes = threads)
    # dispatch = [['MT']]
    for chroms in dispatch:
        genotype_sv_list = dict()
        for chrom in chroms:
            if chrom in svs_tobe_genotyped:
                genotype_sv_list[chrom] = svs_tobe_genotyped[chrom]
        if len(genotype_sv_list) == 0:
            continue
        # pool_result.append(solve_fc(chroms, genotype_sv_list, temporary_dir, max_cluster_bias_dict, threshold_gloab_dict, gt_round))
        fx_para = [(chroms, genotype_sv_list, temporary_dir, max_cluster_bias_dict, threshold_gloab_dict, gt_round)]
        pool_result.append(process_pool.map_async(solve_fc_wrapper, fx_para))
    process_pool.close()
    process_pool.join()

    for x in pool_result:
        result.extend(x.get()[0])
    return result

def solve_fc_wrapper(args):
    return solve_fc(*args)
def solve_fc(chrom_list, svs_dict, temporary_dir, max_cluster_bias_dict, threshold_gloab_dict, gt_round):
    reads_info = dict() # [10000, 10468, 0, 'm54238_180901_011437/52298335/ccs']
    readsfile = open("%sreads.sigs"%(temporary_dir), 'r')
    for line in readsfile:
        seq = line.strip().split('\t')
        chr = seq[0]
        if chr not in chrom_list:
            continue
        if chr not in reads_info:
            reads_info[chr] = list()
        reads_info[chr].append([int(seq[1]), int(seq[2]), int(seq[3]), seq[4]])
    readsfile.close()
    
    sv_dict = dict()
    for sv_type in ["DEL", "DUP", "INS", "INV", "TRA"]:
        sv_dict[sv_type] = parse_sigs_chrom(sv_type, temporary_dir, chrom_list)
    
    gt_list = list()
    for chrom in svs_dict:
        read_id_dict = dict()
        ci_dict = dict()
        search_list = list()
        for i in range(len(svs_dict[chrom])):
            record = svs_dict[chrom][i]
            sv_type = record[0]
            sv_chr2 = record[1]
            sv_start = record[2]
            sv_end = record[3]
            chrom = record[8]
            search_id_list = list()
            # rewrite!
            if sv_type == 'TRA' and 'TRA' in sv_dict and chrom in sv_dict['TRA'] and sv_chr2 in sv_dict['TRA'][chrom]:
                search_id_list = sv_dict['TRA'][chrom][sv_chr2]
            elif sv_type != 'TRA' and sv_type in sv_dict and chrom in sv_dict[sv_type]:
                search_id_list = sv_dict[sv_type][chrom]
            max_cluster_bias = 0
            if sv_type == 'INS' or sv_type == 'DEL':
                read_id_list, max_cluster_bias, CIPOS, CILEN = find_in_indel_list(sv_type, search_id_list, max_cluster_bias_dict[sv_type], sv_start, sv_end, threshold_gloab_dict[sv_type])
            else:
                read_id_list, max_cluster_bias = find_in_list(sv_type, search_id_list, max_cluster_bias_dict[sv_type], sv_start, sv_end)
                CIPOS = '.'
                CILEN = '.'
            
            if sv_type == 'INS':
                max_cluster_bias = max(1000, max_cluster_bias)
            else:
                max_cluster_bias = max(max_cluster_bias_dict[sv_type], max_cluster_bias)
            
            if sv_type == 'INS' or sv_type == 'DEL' or sv_type == 'TRA':
                search_list.append((max(sv_start - max_cluster_bias, 0), sv_start + max_cluster_bias))
            elif sv_type == 'INV' or sv_type == 'DUP':
                # search_list.append((max(sv_start - max_cluster_bias/2, 0), sv_start + max_cluster_bias/2))
                search_list.append((sv_start, sv_end))

            read_id_dict[i] = read_id_list
            ci_dict[i] = (CIPOS, CILEN)

        if chrom in reads_info:
            iteration_dict, primary_num_dict, cover_dict = overlap_cover(search_list, reads_info[chrom]) # both key(sv idx), value(set(read id))
        else:
            iteration_dict = dict()
            primary_num_dict = dict()
            cover_dict = dict()
            for i in read_id_dict:
                iteration_dict[i] = 0
                primary_num_dict[i] = 0
                cover_dict[i] = set()
        assert len(iteration_dict) == len(read_id_dict), "overlap length error"
        assert len(cover_dict) == len(svs_dict[chrom]), "cover length error"
        assign_list = assign_gt(iteration_dict, primary_num_dict, cover_dict, read_id_dict)
        for i in range(len(svs_dict[chrom])):
            assert len(assign_list[i]) == 6, "assign genotype error"
            record = svs_dict[chrom][i]
            rname = ','.join(read_id_dict[i])
            if rname == '':
                rname = 'NULL'
            if record[6] == '<TRA>' or record[6] == '<BND>':
                seq = str(record[1]) + ':' + str(record[3])
            else:
                seq = '<' + record[0] + '>'
            gt_list.append([record[8], record[2], assign_list[i][2], record[0], record[3],
                            ci_dict[i][0], ci_dict[i][1], assign_list[i], rname, record[4],
                            record[5], record[6],
                            record[7], seq])
        logging.info("Finished calling %s."%(chrom))
    return gt_list



import sys
import argparse
import logging
import time

def pase_info(seq):
	info = {'SVLEN': 0, 'END': 0, "SVTYPE": '', "RE": 0, "CHR2": ''}
	for i in seq.split(';'):
		if i.split('=')[0] in ["SVLEN", "END", "RE"]:
			try:
				info[i.split('=')[0]] = abs(int(float(i.split('=')[1])))
			except:
				pass
		if i.split('=')[0] in ["CHR2"]:
			info[i.split('=')[0]] = i.split('=')[1]
		if i.split('=')[0] in ["SVTYPE"]:
			info[i.split('=')[0]] = i.split('=')[1][0:3]

	return info

def load_callset(path, filter=0):
	callset = dict()
	file = open(path, 'r')
	for line in file:
		seq = line.strip('\n').split('\t')
		if seq[0][0] == '#':
			continue

		chr = seq[0]
		pos = int(seq[1])
		info = pase_info(seq[7])

		if info['SVTYPE'] in ['DEL', 'INS', 'DUP', 'INV']:
			if info['SVTYPE'] not in callset:
				callset[info['SVTYPE']] = dict()
			if info['SVLEN'] == 0:
				info['SVLEN'] = info['END'] - pos + 1

			if chr not in callset[info['SVTYPE']]:
				callset[info['SVTYPE']][chr] = list()

			callset[info['SVTYPE']][chr].append([pos, info['END'], info['SVLEN'], [0,0,0]])

		if info['SVTYPE'] == "BND":
			if seq[4][0] == ']':
				form = ']]N'
				chr2 = seq[4].split(':')[0][1:]
				pos2 = int(seq[4].split(':')[1][:-2])
			elif seq[4][0] == '[':
				form = '[[N'
				chr2 = seq[4].split(':')[0][1:]
				pos2 = int(seq[4].split(':')[1][:-2])
			else:
				if seq[4][1] == ']':
					form = 'N]]'
					chr2 = seq[4].split(':')[0][2:]
					pos2 = int(seq[4].split(':')[1][:-1])
				else:
					form = 'N[['
					chr2 = seq[4].split(':')[0][2:]
					pos2 = int(seq[4].split(':')[1][:-1])
			if info['SVTYPE'] not in callset:
				callset[info['SVTYPE']] = dict()
			if info['END'] == 0:
				info['CHR2'] = chr2
				info['END'] = pos2

			if filter > 0:
				if int(seq[-1].split(":")[1].split(',')[1]) < filter:
					continue

			if chr not in callset[info['SVTYPE']]:
				callset[info['SVTYPE']][chr] = list()
			callset[info['SVTYPE']][chr].append([pos, info['CHR2'], info['END'], form, [0,0,0]])

	file.close()
	return callset

def eva_record(call_A, call_B, bias, offect, tag1, tag2):
	# call_A 0/1
	# call_B 1/1
	for svtype in call_A:
		if svtype not in call_B:
			continue

		for chr in call_A[svtype]:
			if chr not in call_B[svtype]:
				continue

			for i in call_A[svtype][chr]:
				for j in call_B[svtype][chr]:

					if svtype == 'INS':
						if abs(i[0]-j[0]) <= offect and float(min(i[2],j[2])/max(i[2],j[2])) >= bias:
							i[-1][tag1] = 1
							j[-1][tag2] = 1
					elif svtype == 'BND':
						if i[1] == j[1] and i[3] == j[3]:
							if abs(i[0]-j[0]) <= offect and abs(i[2]-j[2]) <= offect:
								i[-1][tag1] = 1
								j[-1][tag2] = 1
					else:
						if max(i[0]-offect, j[0]) <= min(i[1]+offect, j[1]) and float(min(i[2],j[2])/max(i[2],j[2])) >= bias:
							i[-1][tag1] = 1
							j[-1][tag2] = 1

def statistics(callset, a, b, c, d):
	for svtype in callset:
		record = 0
		record_000 = 0
		record_100 = 0
		record_010 = 0
		record_001 = 0
		record_110 = 0
		record_101 = 0
		record_011 = 0
		record_111 = 0

		for chr in callset[svtype]:
			for i in callset[svtype][chr]:
				record += 1
				if i[-1][0] == 0 and i[-1][1] == 0 and i[-1][2] == 0:
					record_000 += 1
				if i[-1][0] == 1 and i[-1][1] == 0 and i[-1][2] == 0:
					record_100 += 1
				if i[-1][0] == 0 and i[-1][1] == 1 and i[-1][2] == 0:
					record_010 += 1
				if i[-1][0] == 0 and i[-1][1] == 0 and i[-1][2] == 1:
					record_001 += 1
				if i[-1][0] == 1 and i[-1][1] == 1 and i[-1][2] == 0:
					record_110 += 1
				if i[-1][0] == 1 and i[-1][1] == 0 and i[-1][2] == 1:
					record_101 += 1
				if i[-1][0] == 0 and i[-1][1] == 1 and i[-1][2] == 1:
					record_011 += 1
				if i[-1][0] == 1 and i[-1][1] == 1 and i[-1][2] == 1:
					record_111 += 1
				
		logging.info("%s number of %s:\t%d"%(svtype, a, record))
		logging.info("Only %s:\t%d"%(a, record_000))
		logging.info("%s and %s:\t%d"%(a, b, record_100))
		logging.info("%s and %s:\t%d"%(a, c, record_010))
		logging.info("%s and %s:\t%d"%(a, d, record_001))
		logging.info("%s and %s and %s:\t%d"%(a, b, c, record_110))
		logging.info("%s and %s and %s:\t%d"%(a, b, d, record_101))
		logging.info("%s and %s and %s:\t%d"%(a, c, d, record_011))
		logging.info("%s and %s and %s and %s:\t%d"%(a, b, c, d, record_111))
		logging.info("-----")

def main_ctrl(args):	
	logging.info("Load SV callset of selected caller.")

	cuteSV_callset = load_callset(args.c1)
	# for key in cuteSV_callset:
	# 	print(key, len(cuteSV_callset[key]))
	Sniffles_callset = load_callset(args.c2)
	# for key in Sniffles_callset:
	# 	print(key, len(Sniffles_callset[key]))
	PBSV_callset = load_callset(args.c3, 3)
	# for key in PBSV_callset:
	# 	print(key, len(PBSV_callset[key]))
	SVIM_callset = load_callset(args.c4)
	# for key in SVIM_callset:
	# 	print(key, len(SVIM_callset[key]))

	logging.info("Comparing...")
	eva_record(cuteSV_callset, Sniffles_callset, args.bias, args.offect, 0, 0)
	eva_record(cuteSV_callset, PBSV_callset, args.bias, args.offect, 1, 0)
	eva_record(cuteSV_callset, SVIM_callset, args.bias, args.offect, 2, 0)
	eva_record(Sniffles_callset, PBSV_callset, args.bias, args.offect, 1, 1)
	eva_record(Sniffles_callset, SVIM_callset, args.bias, args.offect, 2, 1)
	eva_record(PBSV_callset, SVIM_callset, args.bias, args.offect, 2, 2)

	logging.info("Final results:")
	statistics(cuteSV_callset, "cuteSV", "Sniffles", "PBSV", "SVIM")
	logging.info("Final results:")
	statistics(Sniffles_callset, "Sniffles", "cuteSV", "PBSV", "SVIM")
	logging.info("Final results:")
	statistics(PBSV_callset, "PBSV", "cuteSV", "Sniffles", "SVIM")
	logging.info("Final results:")
	statistics(SVIM_callset, "SVIM", "cuteSV", "Sniffles", "PBSV")


def main(argv):
	args = parseArgs(argv)
	setupLogging(False)
	# print args
	starttime = time.time()
	main_ctrl(args)
	logging.info("Finished in %0.2f seconds."%(time.time() - starttime))

USAGE="""\
	Evaluate SV callset generated by cuteSV/Sniffles/PBSV/SVIM
"""

def parseArgs(argv):
	parser = argparse.ArgumentParser(prog="Venn_eval", description=USAGE, formatter_class=argparse.RawDescriptionHelpFormatter)
	parser.add_argument("c1", type=str, help="cuteSV callset")
	parser.add_argument('c2', type=str, help = "Sniffles callset")
	parser.add_argument('c3', type=str, help = "PBSV callset")
	parser.add_argument('c4', type=str, help = "SVIM callset")
	parser.add_argument('-b', '--bias', help = "Bias of overlaping.[%(default)s]", default = 0.7, type = float)
	parser.add_argument('-o', '--offect', help = "Offect of translocation overlaping.[%(default)s]", default = 1000, type = int)
	args = parser.parse_args(argv)
	return args

def setupLogging(debug=False):
	logLevel = logging.DEBUG if debug else logging.INFO
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	logging.basicConfig( stream=sys.stderr, level=logLevel, format=logFormat )
	logging.info("Running %s" % " ".join(sys.argv))

if __name__ == '__main__':
	main(sys.argv[1:])


import numpy as np
import logging
from cuteSV.cuteSV_genotype import overlap_cover, assign_gt

def resolution_INV(path, chr, svtype, read_count, max_cluster_bias, sv_size, 
    bam_path, action, MaxSize, gt_round):
    '''
    cluster INV
    ************************************************************************
    path:	INV.sigs
    chr:	chromosome id
    svtype:	<INV>
    
    SEQTYPE		read_count 	max_cluster_bias 	sv_size 
    ------------------------------------------------------------------------
    CCS			5			10 bp (<500 bp)		20 bp 	
    CLR			5			20 bp (<500 bp)		50 bp 	
    ------------------------------------------------------------------------
    
    Input file format
    ------------------------------------------------------------------------
    column	#1	#2	#3	#4	#5
            INV	CHR	BP1	BP2	ID	
    #1	inversion type
    #2	chromosome number
    #3	breakpoint_1 in each read
    #4	breakpoint_2 in each read
    #5	read ID
    ************************************************************************
    '''

    # Initialization of some temporary variables
    semi_inv_cluster = list()
    semi_inv_cluster.append([0,0,'',''])
    candidate_single_SV = list()

    # Load inputs & cluster breakpoint from each signature read 
    file = open("%s%s.sigs"%(path, "INV"), 'r')
    for line in file:
        seq = line.strip('\n').split('\t')
        if seq[1] != chr:
            continue

        strand = seq[2]
        breakpoint_1_in_read = int(seq[3])
        breakpoint_2_in_read = int(seq[4])
        read_id = seq[5]

        # print("new")
        # print(seq[1], seq[2], seq[3], seq[4], seq[5])
        # print(semi_inv_cluster)

        if breakpoint_1_in_read - semi_inv_cluster[-1][0] > max_cluster_bias or breakpoint_2_in_read - semi_inv_cluster[-1][1] > max_cluster_bias or strand != semi_inv_cluster[-1][-1]:
            if len(semi_inv_cluster) >= read_count:
                if semi_inv_cluster[-1][0] == semi_inv_cluster[-1][1] == 0:
                    pass
                else:
                    generate_semi_inv_cluster(semi_inv_cluster, 
                                            chr, 
                                            svtype, 
                                            read_count, 
                                            sv_size, 
                                            candidate_single_SV, 
                                            max_cluster_bias,
                                            action,
                                            MaxSize,
                                            gt_round)
            semi_inv_cluster = []
            semi_inv_cluster.append([breakpoint_1_in_read, breakpoint_2_in_read, read_id, strand])
        else:
            if semi_inv_cluster[-1][0] == semi_inv_cluster[-1][1] == 0:
                semi_inv_cluster = []
                semi_inv_cluster.append([breakpoint_1_in_read, breakpoint_2_in_read, read_id, strand])
            else:
                semi_inv_cluster.append([breakpoint_1_in_read, breakpoint_2_in_read, read_id, strand])

    if len(semi_inv_cluster) >= read_count:
        if semi_inv_cluster[-1][0] == semi_inv_cluster[-1][1] == 0:
            pass
        else:
            generate_semi_inv_cluster(semi_inv_cluster, 
                                    chr, svtype, 
                                    read_count, 
                                    sv_size, 
                                    candidate_single_SV, 
                                    max_cluster_bias,
                                    action,
                                    MaxSize,
                                    gt_round)
    file.close()
    if action:
        candidate_single_SV_gt = call_gt(path, chr, candidate_single_SV, max_cluster_bias)
        logging.info("Finished %s:%s."%(chr, "INV"))
        return candidate_single_SV_gt
    else:
        logging.info("Finished %s:%s."%(chr, "INV"))
        return candidate_single_SV

def generate_semi_inv_cluster(semi_inv_cluster, chr, svtype, read_count, sv_size, 
    candidate_single_SV, max_cluster_bias, action, MaxSize, gt_round):

    strand = semi_inv_cluster[0][-1]

    read_id = [i[2] for i in semi_inv_cluster]
    support_read = len(list(set(read_id)))
    if support_read < read_count:
        return

    inv_cluster_b2 = sorted(semi_inv_cluster, key = lambda x:x[1])

    # breakpoint_1 = np.mean(breakpoint_1_candidate)
    last_bp = inv_cluster_b2[0][1]
    temp_count = 1
    # max_count = 0
    temp_sum_b1 = inv_cluster_b2[0][0]
    temp_sum_b2 = last_bp

    # max_sum = 0
    temp_id = dict()
    temp_id[inv_cluster_b2[0][2]] = 0

    for i in inv_cluster_b2[1:]:
        if i[1] - last_bp > max_cluster_bias:
            if temp_count >= read_count:
                max_count_id = len(temp_id)

                breakpoint_1 = round(temp_sum_b1 / temp_count)
                breakpoint_2 = round(temp_sum_b2 / temp_count)
                inv_len = breakpoint_2 - breakpoint_1
                if inv_len >= sv_size and max_count_id >= read_count:
                    # candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\n'%(chr, svtype, breakpoint_1, breakpoint_2, max_count_id))
                    if inv_len <= MaxSize or MaxSize == -1:
                        if action:
                            candidate_single_SV.append([chr, 
                                                        svtype, 
                                                        breakpoint_1, 
                                                        inv_len, 
                                                        max_count_id,
                                                        strand,
                                                        list(temp_id.keys()),
                                                        breakpoint_2])
                        else:
                            candidate_single_SV.append([chr, 
                                                        svtype, 
                                                        str(int(breakpoint_1)), 
                                                        str(int(inv_len)), 
                                                        str(max_count_id),
                                                        '.',
                                                        './.',
                                                        strand,
                                                        '.,.,.',
                                                        ".",
                                                        ".",
                                                        str(','.join(list(temp_id.keys())))])
                        # print(chr, svtype, str(int(breakpoint_1)), str(int(inv_len)), str(max_count_id), str(DR), str(GT), strand)

            temp_id = dict()
            temp_count = 1
            temp_sum_b1 = i[0]
            temp_sum_b2 = i[1]
            temp_id[i[2]] = 0
        else:
            if i[2] not in temp_id:
                temp_id[i[2]] = 0
            else:
                temp_id[i[2]] += 1
            temp_count += 1
            temp_sum_b1 += i[0]
            temp_sum_b2 += i[1]
        last_bp = i[1]
    if temp_count >= read_count:
        max_count_id = len(temp_id)
        breakpoint_1 = round(temp_sum_b1 / temp_count)
        breakpoint_2 = round(temp_sum_b2 / temp_count)
        inv_len = breakpoint_2 - breakpoint_1
        if inv_len >= sv_size and max_count_id >= read_count:
            # candidate_single_SV.append('%s\t%s\t%d\t%d\t%d\n'%(chr, svtype, breakpoint_1, breakpoint_2, max_count_id))
            if inv_len <= MaxSize or MaxSize == -1:
                if action:
                    candidate_single_SV.append([chr, 
                                                svtype, 
                                                breakpoint_1, 
                                                inv_len, 
                                                max_count_id,
                                                strand,
                                                list(temp_id.keys()),
                                                breakpoint_2])
                else:
                    candidate_single_SV.append([chr, 
                                                svtype, 
                                                str(int(breakpoint_1)), 
                                                str(int(inv_len)), 
                                                str(max_count_id),
                                                '.',
                                                './.',
                                                strand,
                                                '.,.,.',
                                                ".",
                                                ".",
                                                str(','.join(list(temp_id.keys())))])
                # print(chr, svtype, str(int(breakpoint_1)), str(int(inv_len)), str(max_count_id), str(DR), str(GT), strand)

def run_inv(args):
    return resolution_INV(*args)

def call_gt(temporary_dir, chr, candidate_single_SV, max_cluster_bias):
    reads_list = list() # [(10000, 10468, 0, 'm54238_180901_011437/52298335/ccs'), ...]
    readsfile = open("%sreads.sigs"%(temporary_dir), 'r')
    for line in readsfile:
        seq = line.strip().split('\t')
        if seq[0] != chr:
            continue
        reads_list.append([int(seq[1]), int(seq[2]), int(seq[3]), seq[4]])
    readsfile.close()
    svs_list = list()
    for item in candidate_single_SV:
        svs_list.append((max(item[2] - max_cluster_bias/2, 0), item[2] + max_cluster_bias/2))
    for item in candidate_single_SV:
        svs_list.append((max(item[7] - max_cluster_bias/2, 0), item[7] + max_cluster_bias/2))
    iteration_dict, primary_num_dict, cover_dict = overlap_cover(svs_list, reads_list) # both key(sv idx), value(set(read id))
    assert len(cover_dict) == 2 * len(candidate_single_SV), "overlap length error"
    candidate_single_SV_length = len(candidate_single_SV)
    for idx in range(candidate_single_SV_length):
        for item in cover_dict[idx + candidate_single_SV_length]:
            cover_dict[idx].add(item)
    for idx in range(candidate_single_SV_length, candidate_single_SV_length * 2, 1):
        cover_dict.pop(idx)
    assert len(cover_dict) == len(candidate_single_SV), "overlap length error"

    read_id_dict = dict()
    for i in range(len(candidate_single_SV)):
        read_id_dict[i] = candidate_single_SV[i][6]
    assign_list = assign_gt(iteration_dict, primary_num_dict, cover_dict, read_id_dict)
    # [[DV, DR, GT, GL, GQ, QUAL] ...]
    assert len(candidate_single_SV) == len(assign_list), "assign error"
    candidate_single_SV_gt = list()
    for i in range(len(candidate_single_SV)):
        candidate_single_SV_gt.append([candidate_single_SV[i][0], 
                                    candidate_single_SV[i][1], 
                                    str(int(candidate_single_SV[i][2])), 
                                    str(int(candidate_single_SV[i][3])), 
                                    str(candidate_single_SV[i][4]), 
                                    str(assign_list[i][1]),
                                    str(assign_list[i][2]),
                                    candidate_single_SV[i][5],
                                    str(assign_list[i][3]),
                                    str(assign_list[i][4]),
                                    str(assign_list[i][5]),
                                    ','.join(candidate_single_SV[i][6])])
    return candidate_single_SV_gt